<?php
/**
 * Zend Framework (http://framework.zend.com/)
 *
 * @link      http://github.com/zendframework/ZendSkeletonModule for the canonical source repository
 * @copyright Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd New BSD License
 */

namespace Admin\Controller;
use Zend\Session\Container;
use Zend\Mvc\Controller\AbstractActionController;
use Zend\Db\ResultSet\ResultSet;
use Zend\EventManager\EventManagerInterface;

class ReportsController extends AbstractActionController
{
    /**
     * @var config
     */
	protected $config;
    
    /**
     * @var convert
     */
	protected $convert;
    
    /**
     * @var usersTable
     */
    protected $usersTable;
    
    /**
     * @var resourceTable
     */
    protected $resourceTable;
    
    /**
     * Inject an EventManager instance
     *
     * @param  EventManagerInterface $eventManager
     * @return void
     */
    public function setEventManager(EventManagerInterface $events)
    {
        parent::setEventManager($events);
 
        $controller = $this;
        $events->attach('dispatch', function ($e) use ($controller) {
            $controller->createRequestTrackerRecord();
        }, 100); // execute before executing action logic
 
        return $this;
    }
    
    /**
     * Private function to intialize the convert and config variable.
     */
    public function createRequestTrackerRecord()
    {
        // call the convert param service to encryption and decryption as required.
        $this->convert = $this->getServiceLocator()->get('ConvertParams');
        $this->config = $this->getServiceLocator()->get('Config');
    }

        /**
     * Gets an instance of authentication service.
     *
     * @return instance of the service locator
     */
    public function getAuthService()
    {
		// check if the authserive data member is initialized
        if (! $this->authservice) {
            $this->authservice = $this->getServiceLocator()->get('AuthService');
        }
        
        return $this->authservice;
    }
    
    /**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
        // Assign all array of constants to the constantArray variable.
        $constantArray = $this->getServiceLocator()->get('config');

		$this->sessionInstance = new Container($constantArray['SESSION_CONTAINER_NAME']);
		return $this->sessionInstance;
	}
    
    public function indexAction()
    {
         // get all constants and store in constantArray variable.
        $constantArray = $this->config;

		$isLoggedIn = $constantArray['resourceMedication']['DISCONT_MEDICATION_YES'];
        
		// check if the user is already logged in.
		if (! $this->getAuthService()->hasIdentity()) {
            
            $isLoggedIn = $constantArray['resourceMedication']['DISCONT_MEDICATION_NO'];
			return $this->redirect()->toRoute('login');
		}
        
        $session = $this->intializeSessionInstance();
        $isAdmin = $session->offsetGet('is_admin');
        $userId = $session->offsetGet('user_id');
        
        // if the logged in user is not admin then redirect to secure section.
        if (!$isAdmin) {
            return $this->redirect()->toRoute('secure');
        }
        
        $rowSet = $this->getPersonTable()->getUserName($userId);
        
         // If we get the user information then send it to the layouts file.
		if ($rowSet) {
			$this->getEvent()->getViewModel()->username = $rowSet['first_name'] 
				. ' ' . $rowSet['last_name'];
		}
        
        // send data to the layouts file.
        $this->getEvent()->getViewModel()->setVariables(
            array(
                'is_logged_in'           => $isLoggedIn,
                'noBetaFeedbackLink'     => TRUE
            )
        );
        
        $reports = array();
        
        // get an instance of register form
		$request = $this->getRequest();

		// check if the request is a post request
        if ($request->isPost()) {

			// collect the posted data
			$postedData = $this->getRequest()->getPost();
            
            $startDate = '';
            $endDate = '';
            
            if (isset($postedData['start_date']) && !empty($postedData['start_date'])) {
                
                $startDate = date('Y-m-d',strtotime($postedData['start_date']));
            }
            
            if (isset($postedData['end_date']) && !empty($postedData['end_date'])) {
                
                $endDate = date('Y-m-d',strtotime($postedData['end_date']));
            }
            
            // if both start date and end date are set and not empty.
            if ($startDate && $endDate) {
                
                $reports['Users'] = $this
                                        ->getAddedUpdatedUserRecords($startDate, $endDate);
                $reports['Profiles'] = $this
                                            ->getAddedUpdatedProfileRecords(
                                                    $startDate, 
                                                    $endDate
                );
                
                $reports['Contacts'] = $this
                                            ->getAddedUpdatedContactRecords(
                                                    $startDate, 
                                                    $endDate
                );
                
                $reports['Documents'] = $this
                                            ->getAddedUpdatedDocumentRecords(
                                                    $startDate, 
                                                    $endDate
                );
                
                $reports['Events'] = $this
                                        ->getAddedUpdatedEventRecords(
                                                $startDate, 
                                                $endDate
                );
                
                $reports['Medications'] = $this
                                            ->getAddedUpdatedMedicationRecords(
                                                    $startDate, 
                                                    $endDate
                );
                
                $reports['Tasks'] = $this
                                        ->getAddedUpdatedTaskRecords(
                                                $startDate, 
                                                $endDate
                );
                
                $reports['Posts'] = $this
                                        ->getAddedUpdatedPostRecords(
                                                $startDate, 
                                                $endDate
                );
            }
            
            return array(
                'activityArray' => $reports,
                'postedData'    => $postedData
            );
        }
    }
    
    /**
     * Function to count all created and modified records from users table.
     * 
     * @param string $startDate
     * @param string $endDate
     * @return array and return false on failure.
     */
    private function getAddedUpdatedUserRecords($startDate, $endDate)
    {
        $userCountArray = array();
        
        // check if we have start date and end date.
        if ($startDate && $endDate) {
            
            // collect the no. of users that were registered within start date and end date.
            $queryResultForUsersCreated = $this
                                            ->getUsersTable()
                                            ->countUsersCreatedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            $queryResultForUsersModified = $this
                                            ->getUsersTable()
                                            ->countUsersModifiedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            foreach($queryResultForUsersCreated as $row) {

                $userCountArray['added'] = $row->noOfcreatedUsers;
            }

            foreach($queryResultForUsersModified as $row) {

                $userCountArray['updated'] = $row->noOfmodifiedUsers;
            }
            
            return $userCountArray;
        }
        
        return FALSE;
    }
    
    /**
     * Function to count all created and modified records from users table.
     * 
     * @param string $startDate
     * @param string $endDate
     * @return array and return false on failure.
     */
    private function getAddedUpdatedProfileRecords($startDate, $endDate)
    {
        $profileCountArray = array();
        
        // check if we have start date and end date.
        if ($startDate && $endDate) {
            
            $noOfProfilesCreated = $this
                                        ->getResourceTable()
                                        ->countProfilesCreatedWithinDates(
                                                $startDate, 
                                                $endDate
            );

            $noOfProfilesUpdated = $this
                                        ->getResourceTable()
                                        ->countProfilesModifiedWithinDates(
                                                $startDate, 
                                                $endDate
            );

            $profileCountArray['added'] = $noOfProfilesCreated;
            $profileCountArray['updated'] = $noOfProfilesUpdated;
            
            return $profileCountArray;
        }
        
        return FALSE;
    }
    
    /**
     * Function to count all created and modified records from users table.
     * 
     * @param string $startDate
     * @param string $endDate
     * @return array and return false on failure.
     */
    private function getAddedUpdatedContactRecords($startDate, $endDate)
    {
        $contactCountArray = array();
        
        // check if we have start date and end date.
        if ($startDate && $endDate) {
            
            $queryResultForContactsCreated = $this
                                            ->getResourceTable()
                                            ->countContactsCreatedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            $queryResultForContactsModified = $this
                                            ->getResourceTable()
                                            ->countContactsModifiedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            foreach($queryResultForContactsCreated as $row) {

                $contactCountArray['added'] = $row->noOfcreatedContacts;
            }

            foreach($queryResultForContactsModified as $row) {

                $contactCountArray['updated'] = $row->noOfmodifiedContacts;
            }
            
            return $contactCountArray;
        }
        
        return FALSE;
    }
    
    /**
     * Function to count all created and modified records from users table.
     * 
     * @param string $startDate
     * @param string $endDate
     * @return array and return false on failure.
     */
    private function getAddedUpdatedDocumentRecords($startDate, $endDate)
    {
        $documentCountArray = array();
        
        // check if we have start date and end date.
        if ($startDate && $endDate) {
            
            $queryResultForDocumentsCreated = $this
                                            ->getResourceTable()
                                            ->countDocumentsCreatedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            $queryResultForDocumentsModified = $this
                                            ->getResourceTable()
                                            ->countDocumentsModifiedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            foreach($queryResultForDocumentsCreated as $row) {

                $documentCountArray['added'] = $row->noOfcreatedDocuments;
            }

            foreach($queryResultForDocumentsModified as $row) {

                $documentCountArray['updated'] = $row->noOfmodifiedDocuments;
            }
            
            return $documentCountArray;
        }
        
        return FALSE;
    } 
    
    /**
     * 
     * 
     * @param type $startDate
     * @param type $endDate
     */
    private function getAddedUpdatedEventRecords($startDate, $endDate)
    {
        $eventCountArray = array();
        
        // check if we have start date and end date.
        if ($startDate && $endDate) {
            
            $queryResultForEventsCreated = $this
                                            ->getResourceTable()
                                            ->countEventsCreatedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            $queryResultForEventsModified = $this
                                            ->getResourceTable()
                                            ->countEventsModifiedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            foreach($queryResultForEventsCreated as $row) {

                $eventCountArray['added'] = $row->noOfcreatedEvents;
            }

            foreach($queryResultForEventsModified as $row) {

                $eventCountArray['updated'] = $row->noOfmodifiedEvents;
            }
            
            return $eventCountArray;
        }
        
        return FALSE;
    }
    
    /**
     * 
     * 
     * @param type $startDate
     * @param type $endDate
     */
    private function getAddedUpdatedMedicationRecords($startDate, $endDate)
    {
        $medicationCountArray = array();
        
        // check if we have start date and end date.
        if ($startDate && $endDate) {
            
            $queryResultForMedicationsCreated = $this
                                                ->getResourceTable()
                                                ->countMedicationsCreatedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            $queryResultForMedicationsModified = $this
                                            ->getResourceTable()
                                            ->countMedicationsModifiedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            foreach($queryResultForMedicationsCreated as $row) {

                $medicationCountArray['added'] = $row->noOfcreatedMedications;
            }

            foreach($queryResultForMedicationsModified as $row) {

                $medicationCountArray['updated'] = $row->noOfmodifiedMedications;
            }
            
            return $medicationCountArray;
        }
        
        return FALSE;
    }
    
    /**
     * 
     * 
     * @param type $startDate
     * @param type $endDate
     */
    private function getAddedUpdatedPostRecords($startDate, $endDate)
    {
        $postCountArray = array();
        
        // check if we have start date and end date.
        if ($startDate && $endDate) {
            
            $queryResultForPostsCreated = $this
                                            ->getResourceTable()
                                            ->countPostsCreatedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            $queryResultForPostsModified = $this
                                            ->getResourceTable()
                                            ->countPostsModifiedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            foreach($queryResultForPostsCreated as $row) {

                $postCountArray['added'] = $row->noOfcreatedPosts;
            }

            foreach($queryResultForPostsModified as $row) {

                $postCountArray['updated'] = $row->noOfmodifiedPosts;
            }
            
            return $postCountArray;
        }
        
        return FALSE;
    }
    
    /**
     * Function to get all added and updated task records.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    private function getAddedUpdatedTaskRecords($startDate, $endDate)
    {
        $taskCountArray = array();
        
        // check if we have start date and end date.
        if ($startDate && $endDate) {
            
            $queryResultForTasksCreated = $this
                                            ->getResourceTable()
                                            ->countTasksCreatedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            $queryResultForTasksModified = $this
                                            ->getResourceTable()
                                            ->countTasksModifiedWithinDates(
                                                    $startDate, 
                                                    $endDate
            );

            foreach($queryResultForTasksCreated as $row) {

                $taskCountArray['added'] = $row->noOfcreatedTasks;
            }

            foreach($queryResultForTasksModified as $row) {

                $taskCountArray['updated'] = $row->noOfmodifiedTasks;
            }
            
            return $taskCountArray;
        }
        
        return FALSE;
    }
    
    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getPersonTable()
	{
		// check if usersTable previously defined.
		if (! $this->personTable) {
			$sm = $this->getServiceLocator();
			$this->personTable = $sm->get('Secure\Model\PersonTable');
		}
		return $this->personTable;
	}

    /**
     * Get an instance of resourceTable by using service manager.
     *
     * @return service manager instance for resource table.
     */
	public function getResourceTable()
	{
		// check if usersTable previously defined.
		if (! $this->resourceTable) {
			$sm = $this->getServiceLocator();
			$this->resourceTable = $sm->get('Admin\Model\ResourceTable');
		}
		return $this->resourceTable;
	}
    
    /**
     * Get an instance of usersTable by using service manager.
     * 
     * @return instance instance of UsersTable model.
     * 
     */
	public function getUsersTable()
	{
		// check if usersTable previously defined.
		if (! $this->usersTable) {
			$sm = $this->getServiceLocator();
			$this->usersTable = $sm->get('Admin\Model\UsersTable');
		}
		return $this->usersTable;
	}
}