<?php

//module/Secure/src/Secure/Model/ResourceTable.php

/**************
	Purpose: Model functions for accessing the resource table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Admin\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert, Zend\Db\Sql\Where, Zend\Db\Sql\Predicate;
use Zend\Db\Sql\Select;
use Zend\Db\Adapter\Adapter;
use Zend\Db\Sql\Expression;

class ResourceTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	/**
     * @var sessionInstance
     */
	private $sessionInstance;
	
	/**
     * @var eventsTable
     */
	protected $eventsTable;
	
	/**
     * @var dbSql
     */
	protected $dbSql;
    
	/**
     * @var table
     */
	protected $table = 'resource';
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countProfilesCreatedWithinDates($startDate, $endDate)
    {
        // if startDate and endDate and set.
        if ($startDate && $endDate) {
            
            $noOfCaredforCreated = 0;
            $noOfProviderCreated = 0;
            
            try {
                // select all the person records within the given date range.
                $selectFromPerson = new Select();
                $selectFromPerson->columns(array('noOfCreatedPersons' => new Expression('COUNT(personid)')));
                $selectFromPerson->from('person');
                $selectFromPerson->where("date( created ) >= date '" . $startDate . "'");
                $selectFromPerson->where("date( created ) <= date '" . $endDate . "'");
                
                // select all the provider records within the given date range.
                $selectFromProvider = new Select();
                $selectFromProvider->columns(array('noOfCreatedProviders' => new Expression('COUNT(providerid)')));
                $selectFromProvider->from('provider');
                $selectFromProvider->where("date( created ) >= date '" . $startDate . "'");
                $selectFromProvider->where("date( created ) <= date '" . $endDate . "'");
                
                $personInstance = new TableGateway('person', $this->dbSql);
                $resultSetForCaredfor = $personInstance->selectWith($selectFromPerson);
                
                foreach ($resultSetForCaredfor As $caredFor) {
                    $noOfCaredforCreated = $caredFor['noOfCreatedPersons'];
                }
                
                $providerInstance = new TableGateway('provider', $this->dbSql);
                $resultSetForProvider = $providerInstance->selectWith($selectFromProvider);
                
                foreach ($resultSetForProvider As $provider) {
                    $noOfProviderCreated = $provider['noOfCreatedProviders'];
                }
                
                return $noOfCaredforCreated + $noOfProviderCreated;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of profiles between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countProfilesModifiedWithinDates($startDate, $endDate)
    {
        // if startDate and endDate and set.
        if ($startDate && $endDate) {
            
            $noOfCaredforUpdated = 0;
            $noOfProviderUpdated = 0;
            
            try {
                // select all the person records within the given date range.
                $selectFromPerson = new Select();
                $selectFromPerson->columns(
                        array(
                            'noOfUpdatedPersons' => new Expression('COUNT(personid)')
                        )
                );
                $selectFromPerson->from('person');
                $selectFromPerson->where("date( updated ) >= date '" . $startDate . "'");
                $selectFromPerson->where("date( updated ) <= date '" . $endDate . "'");
                
                // select all the provider records within the given date range.
                $selectFromProvider = new Select();
                $selectFromProvider->columns(
                        array(
                            'noOfUpdatedProviders' => new Expression('COUNT(providerid)')
                        )
                );
                $selectFromProvider->from('provider');
                $selectFromProvider->where("date( updated ) >= date '" . $startDate . "'");
                $selectFromProvider->where("date( updated ) <= date '" . $endDate . "'");
                
                $personInstance = new TableGateway('person', $this->dbSql);
                $resultSetForCaredfor = $personInstance->selectWith($selectFromPerson);
                
                foreach ($resultSetForCaredfor As $caredFor) {
                    $noOfCaredforUpdated = $caredFor['noOfUpdatedPersons'];
                }
                
                $providerInstance = new TableGateway('provider', $this->dbSql);
                $resultSetForProvider = $providerInstance->selectWith($selectFromProvider);
                
                foreach ($resultSetForProvider As $provider) {
                    $noOfProviderUpdated = $provider['noOfUpdatedProviders'];
                }
                
                return $noOfCaredforUpdated + $noOfProviderUpdated;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countContactsCreatedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfcreatedContacts' => new Expression('COUNT(contactid)')));
                $select->from('contact');
                $select->where("date( created ) >= date '" . $startDate . "'");
                $select->where("date( created ) <= date '" . $endDate . "'");
                
                $contactInstance = new TableGateway('contact', $this->dbSql);

                $resultSet = $contactInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countTasksCreatedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfcreatedTasks' => new Expression('COUNT(taskid)')));
                $select->from('task');
                $select->where("date( created ) >= date '" . $startDate . "'");
                $select->where("date( created ) <= date '" . $endDate . "'");
                
                $taskInstance = new TableGateway('task', $this->dbSql);

                $resultSet = $taskInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of profiles between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countTasksModifiedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfmodifiedTasks' => new Expression('COUNT(taskid)')));
                $select->from('task');
                $select->where("date( updated ) >= date '" . $startDate . "'");
                $select->where("date( updated ) <= date '" . $endDate . "'");
                $taskInstance = new TableGateway('task', $this->dbSql);

                $resultSet = $taskInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of profiles between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countContactsModifiedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfmodifiedContacts' => new Expression('COUNT(contactid)')));
                $select->from('contact');
                $select->where("date( updated ) >= date '" . $startDate . "'");
                $select->where("date( updated ) <= date '" . $endDate . "'");
                $contactInstance = new TableGateway('contact', $this->dbSql);

                $resultSet = $contactInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countDocumentsCreatedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfcreatedDocuments' => new Expression('COUNT(documentid)')));
                $select->from('document');
                $select->where("date( created ) >= date '" . $startDate . "'");
                $select->where("date( created ) <= date '" . $endDate . "'");

                $documentInstance = new TableGateway('document', $this->dbSql);
                $resultSet = $documentInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of profiles between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countDocumentsModifiedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfmodifiedDocuments' => new Expression('COUNT(documentid)')));
                $select->from('document');
                $select->where("date( updated ) >= date '" . $startDate . "'");
                $select->where("date( updated ) <= date '" . $endDate . "'");

                $documentInstance = new TableGateway('document', $this->dbSql);
                $resultSet = $documentInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countEventsCreatedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfcreatedEvents' => new Expression('COUNT(eventid)')));
                $select->from('events');
                $select->where("date( created ) >= date '" . $startDate . "'");
                $select->where("date( created ) <= date '" . $endDate . "'");

                $eventsInstance = new TableGateway('events', $this->dbSql);
                $resultSet = $eventsInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of profiles between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countEventsModifiedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfmodifiedEvents' => new Expression('COUNT(eventid)')));
                $select->from('events');
                $select->where("date( updated ) >= date '" . $startDate . "'");
                $select->where("date( updated ) <= date '" . $endDate . "'");

                $eventsInstance = new TableGateway('events', $this->dbSql);
                $resultSet = $eventsInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countMedicationsCreatedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfcreatedMedications' => new Expression('COUNT(medicationid)')));
                $select->from('medication');
                $select->where("date( created ) >= date '" . $startDate . "'");
                $select->where("date( created ) <= date '" . $endDate . "'");

                $medicationInstance = new TableGateway('medication', $this->dbSql);
                $resultSet = $medicationInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of profiles between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countMedicationsModifiedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfmodifiedMedications' => new Expression('COUNT(medicationid)')));
                $select->from('medication');
                $select->where("date( updated ) >= date '" . $startDate . "'");
                $select->where("date( updated ) <= date '" . $endDate . "'");
                
                $medicationInstance = new TableGateway('medication', $this->dbSql);
                $resultSet = $medicationInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countPostsCreatedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfcreatedPosts' => new Expression('COUNT(postid)')));
                $select->from('post');
                $select->where("date( created ) >= date '" . $startDate . "'");
                $select->where("date( created ) <= date '" . $endDate . "'");

                $postInstance = new TableGateway('post', $this->dbSql);
                $resultSet = $postInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of profiles between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countPostsModifiedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfmodifiedPosts' => new Expression('COUNT(postid)')));
                $select->from('post');
                $select->where("date( updated ) >= date '" . $startDate . "'");
                $select->where("date( updated ) <= date '" . $endDate . "'");

                
                $postInstance = new TableGateway('post', $this->dbSql);
                $resultSet = $postInstance->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
}