<?php
//module/Secure/src/Secure/Model/UsersTable.php

/**************
	Purpose: Model functions for accessing the users table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Admin\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert, Zend\Db\Sql\Where;
use Zend\Db\Sql\Select;
use Zend\Db\Sql\Expression;

class UsersTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	/**
     * @var sessionInstance
     */
	private $sessionInstance;
	
	/**
     * @var eventsTable
     */
	protected $eventsTable;
	
	/**
     * @var dbSql
     */
	protected $dbSql;
    
	/**
     * @var table
     */
	private $table = 'users';

    /**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countUsersCreatedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfcreatedUsers' => new Expression('COUNT(userid)')));
                $select->from($this->table);
                $select->where("date( created ) >= date '" . $startDate . "'");
                $select->where("date( created ) <= date '" . $endDate . "'");


                $resultSet = $this->tableGateway->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
    
    /**
     * Count no. of users between the given start date and end date.
     * 
     * @param type $startDate
     * @param type $endDate
     */
    public function countUsersModifiedWithinDates($startDate, $endDate)
    {
        if ($startDate && $endDate) {
            
            try {
                $select = new Select();
                $select->columns(array('noOfmodifiedUsers' => new Expression('COUNT(userid)')));
                $select->from($this->table);
                $select->where("date( updated ) >= date '" . $startDate . "'");
                $select->where("date( updated ) <= date '" . $endDate . "'");


                $resultSet = $this->tableGateway->selectWith($select);
                return $resultSet;
            }
            catch (\Exception $e) {
                
                return false;
            }
        }
        
        return false;
    }
}