<?php
//module/Secure/src/Secure/Controller/RegisterController.php
/**************
	Purpose: Handels the registration of users into HTC.
	Author: Kevin Gyberg
	Date: 02/26/2013
*************/

namespace Login\Controller;
use Zend\Mvc\Controller\AbstractActionController;
use Zend\EventManager\EventManagerInterface;
use Zend\View\Model\ViewModel;
use Login\Model\UsersTable;
use Login\Form\RegisterForm;
use Zend\Session\Container;
use Zend\Filter\StripTags;
use Zend\Filter\StringTrim;
use Zend\Validator\EmailAddress;
use Zend\Validator\Db\RecordExists;
use Zend\Mail;
use Zend\Mime\Part as MimePart;
use Zend\Mime\Message as MimeMessage; 

class RegisterController extends AbstractActionController
{
	/**
     * @var authservice
     */
	protected $authservice;
    
    /**
     * @var config
     */
	protected $config;

	/**
     * @var authservice
     */
	protected $usersTable;
    
    /**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
        // Assign all array of constants to the constantArray variable.
        $constantArray = $this->config;

		$this->sessionInstance = new Container($constantArray['SESSION_CONTAINER_NAME']);
		return $this->sessionInstance;
	}

	/**
     * Gets an instance of authentication service.
     * @return instance of authservice
     */
    public function getAuthService()
    {
		// check if the authserive data member is initialized
        if (! $this->authservice) {

            $this->authservice = $this->getServiceLocator()->get('AuthService');
        }
        
        return $this->authservice;
    }
    
    /**
     * Inject an EventManager instance
     *
     * @param  EventManagerInterface $eventManager
     * @return void
     */
    public function setEventManager(EventManagerInterface $events)
    {
        parent::setEventManager($events);
 
        $controller = $this;
        $events->attach('dispatch', function ($e) use ($controller) {
            
            $requestTrackerArray = $controller->createRequestTrackerRecord();
            
            $controller
                ->getRequestTrackerTable()
                ->saveRequestTrackerRecord($requestTrackerArray);
            
        }, 100); // execute before executing action logic
 
        return $this;
    }
    
    /**
     * Function to process the request the
     * 
     * @return type
     */
    public function createRequestTrackerRecord()
    {
        // initialize variables to be used in this function.
        $requestUriArray = array();
        
        // array to return after processing
        $requestTrackerArray = array();
        
        // call the convert param service to encryption and decryption as required.
        $this->convert = $this->getServiceLocator()->get('ConvertParams');
        $this->config = $this->getServiceLocator()->get('Config');
        
        // get the constant array
        $constantArray = $this->config;
        
        // initialize the session array.
        $session = $this->intializeSessionInstance();
        $isLoggedIn = $session->offsetGet('is_logged_in');
        
        // check if the request uri key of $_SERVER is not empty.
        if (! empty($_SERVER['REQUEST_URI'])) {
                
            $requestUriArray = explode('/', $_SERVER['REQUEST_URI']);
            
            if (count($requestUriArray) === 2) {
                $requestTrackerArray['resource_action'] 
                                            = $constantArray['account']['ADD_ACTION'];
            }
            
            $requestTrackerArray['http_user_agent'] = $_SERVER['HTTP_USER_AGENT'];
            $requestTrackerArray['script_uri'] = $_SERVER['SCRIPT_NAME'];
            $requestTrackerArray['request_uri'] = $_SERVER['REQUEST_URI'];
            $requestTrackerArray['domain_name'] = $_SERVER['SERVER_NAME'];
            $requestTrackerArray['remote_ipadd'] = $_SERVER['REMOTE_ADDR'];
            $requestTrackerArray['http_referrer'] =  $_SERVER['HTTP_REFERER'];
            
            if ($isLoggedIn === $constantArray['IS_LOGGED_IN_YES']) {

                $requestTrackerArray['session_userid'] = $session->offsetGet('user_id');
            }
            
            $parseArray = parse_url($_SERVER['HTTP_REFERER']);
            $requestTrackerArray['ipaddr_hostname'] = gethostbyname($parseArray['host']);
            $requestTrackerArray['created'] = date('Y-m-d H:i:s');
            $requestTrackerArray['created_date'] = date('Y/m/d', time());
            $requestTrackerArray['created_time'] = date('H:i:s', time());
        }
        
        return $requestTrackerArray;
    }

	/**
     * Renders the register form and processes the posted data
     *
     * @return array(Registration for instance).
     */
	public function indexAction()
	{
        $constantArray = $this->config;
		$isLoggedIn = $constantArray['IS_LOGGED_IN_NO'];
        
		// check if the user is already logged in.
		if ($this->getAuthService()->hasIdentity()) {

			return $this->redirect()->toRoute('secure');
		}

        $this->getEvent()->getViewModel()->setVariables(
                array(
                    'is_logged_in'           => $isLoggedIn,
                    'noBetaFeedbackLink'     => TRUE
                )
        );

        $activationIdString = $this->params()->fromRoute('activationid', 0);

        $decodedActivationString = '';
        
        if ($activationIdString) {

           $convert = $this->getServiceLocator()->get('ConvertParams');
           
           $decodedString = $convert->paramDecrypt($activationIdString);
           $decodedArray = $convert->stringToArray($decodedString);
           
           $decodedActivationString = $decodedArray['emailId'];

           $emailExists = $this
                            ->getUsersTable()
                            ->ifEmailIdExists($decodedActivationString);

           if ($emailExists) {

                $this
                    ->flashmessenger()
                    ->addMessage("Your login has been validated.");
                return $this->redirect()->toRoute('login');
           }
           
           $emailExistsInactive = $this
                                    ->getUsersTable()
                                    ->ifEmailIdExistsInactive(
                                            $decodedActivationString
           );

           if ($emailExistsInactive) {

               $this->flashmessenger()->addMessage("Your email address is 
                            registered in Help To Care but not yet activated.");
               return $this->redirect()->toRoute('login');
           }
        }

		// get an instance of register form
		$request = $this->getRequest();
        
		$captchaService = $this->getServiceLocator()->get('SanCaptcha');
        
        $form = new RegisterForm($captchaService);

		// check if the request is a post request

        if ($request->isPost()) {

			// collect the posted data
			$postedData = $this->getRequest()->getPost();
            
            $form->setData($request->getPost());

			$errorMessages['messages'] = array();
			$isFormValidated = true;
			$userInsertdata = array();
            
			$userInsertdata['password'] = $this
                                            ->filterFormElements(
                                                    $postedData['password']
            );

            $config = $this->getServiceLocator()->get('Config');

			if (! $userInsertdata['password']) {

				$isFormValidated = false;
				$errorMessages['messages']['password']['required'] 
                                                    = 'Please enter Password.';
			}
            else if (strlen($userInsertdata['password']) 
                                        < $config['minimum_password_char']) {

                $isFormValidated = false;
				$errorMessages['messages']['password']['invalidLength'] 
                            = 'Please enter Password minimum of 6 characters.';
            }
            
			$userInsertdata['confPassword'] = $this
                                                ->filterFormElements(
                                                    $postedData["conf_password"]
            );

			if (! $userInsertdata['confPassword']) {

				$isFormValidated = false;
				$errorMessages['messages']['confPassword']['required'] 
                                            = 'Please enter Confirm Password.';
			}
            
			if ($userInsertdata['password'] 
                    && $userInsertdata['confPassword'] 
                    && ! $this->validateConfirmPassword(
                            $userInsertdata['password'], 
                            $userInsertdata['confPassword']
                    )) {

				$isFormValidated = false;
				$errorMessages['messages']['confPassword']['identical'] 
                                        = 'Password and Confirm Password do not match.';
			}
            
			$userInsertdata['firstName'] = $this->filterFormElements(
                    $postedData['first_name']
            );

			if (! $userInsertdata['firstName']) {

				$isFormValidated = false;
				$errorMessages['messages']['firstName']['required'] 
                                                = 'Please enter First Name.';
			}
            
			$userInsertdata['lastName'] = $this
                                ->filterFormElements($postedData['last_name']);

			if (! $userInsertdata['lastName']) {

				$isFormValidated = false;
				$errorMessages['messages']['lastName']['required'] 
                                                    = 'Please enter Last Name.';
			}

			/*
				Codes for captcha validation here.
			*/
			$userInsertdata['termsAgree'] = $this
                    ->filterFormElements($postedData['terms_conditions_agree']);
            
			if (! $userInsertdata['termsAgree']) {

				$isFormValidated = false;
				$errorMessages['messages']['termsAgree']['required'] 
                            = 'Please agree to the Terms of Service and Privacy Policy.';
			}

            // Check that the currently registering user is a spammer.
//            $forumSpamInstance = $this
//                                    ->getServiceLocator()
//                                    ->get('StopForumSpam');
//
//            $username = $userInsertdata['firstName'] 
//                                            . ' ' . $userInsertdata['lastName'];
//
//            $isSpammer = $forumSpamInstance->isSpammer(
//                array(
//                    'email'     => $userInsertdata['loginEmail'],
//                    'ip'        => $_SERVER['REMOTE_ADDR'],
//                    'username'  => $username
//                )
//            );
//
//            // check if the registering user is a spammer
//            if ($isSpammer['spammer']) {
//
//                $isFormValidated = false;
//
//                // send validations messages from here.
//                $errorMessages['messages']['spammer']['required'] 
//                    = 'Your name, IP and email has been listed in spammers list.';
//
//                /* if the spammer is not known to stop forum spam then add him 
//                 to the database.*/
//                if (!$isSpammer['known']) {
//
//                    $forumSpamInstance->add(
//                            array(
//                                'email'     => $userInsertdata['loginEmail'],
//                                'ip'        => $_SERVER['REMOTE_ADDR'],
//                                'username'  => $username
//                            )
//                    );
//                }
//            }
            
            
            $userInsertdata['loginEmail'] = $this
                                                ->filterFormElements(
                                                        $postedData['email_login']
            );

			if (! $userInsertdata['loginEmail']) {

				$isFormValidated = false;
				$errorMessages['messages']['loginEmail']['required'] 
                                        = 'Please enter your E-mail address.';
			}
            
			$emailvalidator = new EmailAddress();

			if ($userInsertdata['loginEmail'] 
                    && ! $emailvalidator->isValid($userInsertdata['loginEmail'])) {

				$isFormValidated = false;
				$errorMessages['messages']['loginEmail']['invalid'] 
                                    = 'Please enter a valid E-mail address.';
			}
			else {
                
                // if the entered email address is valid
                $userInsertdata['loginEmail'] 
                                    = strtolower($userInsertdata['loginEmail']);

                $emailExists = $this
                                    ->getUsersTable()
                                    ->ifEmailIdExists(
                                            $userInsertdata['loginEmail']
                );
                
                // if email id already exists
                if ($emailExists) {
                    
                    $isFormValidated = false;
                    $errorMessages['messages']['loginEmail']['exists'] 
                        = 'The e-mail address you entered has already been registered.';
                }
                else {
                    
                    // check if the email id is present in in-active state.
                    $emailExistsInactive = $this
                                                ->getUsersTable()
                                                ->ifEmailIdExistsInactive(
                                                    $userInsertdata['loginEmail']
                    );
                    
                    if ($emailExistsInactive && $isFormValidated) {
                        
                        $userIdArray = array(
                            'userid' => $emailExistsInactive['userid']
                        );
                    
                        $encryptedUserId = $this
                                            ->createEncodedString($userIdArray);
                        
                        $isFormValidated = false;
                        $errorMessages['messages']['ifEmailIdExistsInactive'] 
                            = 'The entered E-mail address is already registered but 
                            not yet activated. Please <a href="' . $config['base_url'] 
                            . 'login?activation-code=' . $encryptedUserId 
                            . '" style="text-decoration:underline">
                                click here</a> to resend an activation email.';
                    }
                }
            }
           
            $userInsertdata['captchaInput'] = $this
                                                ->filterFormElements(
                                                $postedData['captcha']['input']
            );
            
            if (! $userInsertdata['captchaInput']) {
                
                $isFormValidated = false;
                $errorMessages['messages']['captcha']['required'] 
                                = 'Please enter the captcha text in input box.';
            }
            else {
                // if the user has entered captcha text
                if (! $form->isValid()) {
                    
                    $isFormValidated = false;
                    $errorMessages['messages']['captcha']['invalid'] 
                            = 'Please enter correct text in captcha input box.';
                }
            }
            
			// apply validations on the submitted form
            if ($isFormValidated) {

                // create a random code for 
                $activationCode = sha1($userInsertdata['loginEmail'] . time() 
                        . rand()
                );

                // Also send the activation code to the users model in order to 
                // store it.
                $userInsertdata['activationCode'] = $activationCode;
				
				// save the user data to the users table.
                if (isset($postedData['email_address']) 
                        && ! empty($postedData['email_address'])) {

                    $lastUserIdInsert = $this
                                            ->getUsersTable()
                                            ->saveUser(
                                                    $userInsertdata, 
                                                    $constantArray['IS_LOGGED_IN_YES']
                    );
                }
                else {
                    
                    $lastUserIdInsert = $this
                                            ->getUsersTable()
                                            ->saveUser($userInsertdata);
                }
                
                if ($lastUserIdInsert) {
                    // we need to send an email if and only if 

                    if (! $postedData['email_address']) {

                        // send an email with the generated activation code to the 
                        // user's email id.
                        $convert = $this->getServiceLocator()->get('SendEmail');
                        $activationLink = $config['activation_email_link'] 
                            . $activationCode;

                        // prepare the view for activation email template
                        $mailData = array(
                           'activationLink'     =>      $activationLink,
						   'baseUrl'			=> 		$config['base_url']
                        );

                        $this->renderer = $this
                                            ->getServiceLocator()
                                            ->get('ViewRenderer');
                        
                        $content = $this
                                        ->renderer
                                        ->render(
                                                'activation-email/view', 
                                                $mailData
                        );

                        // create the link for activation code.
                        $smtpOptions = $config['mailer']['smtp_options'];
                        $htmlBody = $content;
                        $subject 
                            = $config['mailer']['registration_mail_constants']['registration_email_subject'];
                        $fromName 
                            = $config['mailer']['registration_mail_constants']['registration_email_from_name'];
                        $fromEmail 
                            = $config['mailer']['registration_mail_constants']['registration_email_from_email'];
                        $to = $userInsertdata['loginEmail'];

                        // $htmlBody, $subject, $fromName, $fromEmail, $to
                        $convert->sendEmail(
                            $smtpOptions, 
                            $htmlBody, 
                            $subject, 
                            $fromName, 
                            $fromEmail, 
                            $to
                        );
                        return $this->redirect()->toRoute(
                            'register',
                            array(
                                'controller' => 'register',
                                'action' => 'goto-login'
                            )
                        );
                    }

                    $this
                        ->getUsersPermTempTable()
                        ->copyPermissions(
                                $userInsertdata['loginEmail'], 
                                $lastUserIdInsert
                    );

					return $this->redirect()->toRoute('login');
				}

				// Unable to register user.
				$errorMessages['messages']['registerStatus']['error'] 
                                                = 'Unable to register user.';
            }

            if (isset($postedData['email_address']) 
                    && ! empty($postedData['email_address'])) {

                return array(
                    'messages'      => $errorMessages['messages'],
                    'setdata'       => $userInsertdata,
                    'email_login'   => $postedData['email_address'],
					'captcha' 		=> $captcha,
                    'base_url'      => $config['base_url'],
                    'form'          => $form
                );
            }
            else {
                return array(
                    'messages'      => $errorMessages['messages'],
                    'setdata'       => $userInsertdata,
					'captcha' 		=> $captcha,
                    'base_url'      => $config['base_url'],
                    'form'          => $form
                );
            }
        }

        $registerReturnArray = array(
            'captcha'   => $captcha,
            'base_url'  => $config['base_url'],
            'form'      => $form
        );

        if ($activationIdString) {

            $registerReturnArray['email_login'] = $decodedActivationString;
        }
        
        return $registerReturnArray;
	}

    /**
     * Function to generate the captcha element.
     * 
     * @return NULL
     */
    public function generateAction()
    {
        // Assign all array of constants to the constantArray variable.
        $constantArray = $this->config;
        
        $response = $this->getResponse();
        
        
        $response->getHeaders()->addHeaderLine('Content-Type', "image/png");
 
        $id = $this->params('id', false);
 
        if ($id) {
 
            $image = $constantArray['CAPTCHA_FOLDER_PATH'] . $id;
 
            if (file_exists($image) !== false) {
                $imagegetcontent = @file_get_contents($image);
 
                $response->setStatusCode(200);
                $response->setContent($imagegetcontent);
 
                if (file_exists($image) == true) {
                    unlink($image);
                }
            }
 
        }
 
        return $response;
    }

    /**
     * Provide users the link for redirecting towards the login page.
     * 
     * @return NULL
     */
    public function gotoLoginAction()
	{
        // Assign all group of constants to the constantArray variable.
        $constantArray = $this->config;
        
        $isLoggedIn = $constantArray['IS_LOGGED_IN_NO'];

		// check if the user is already logged in.
		if ($this->getAuthService()->hasIdentity()) {

			$isLoggedIn = $constantArray['IS_LOGGED_IN_YES'];
			return $this->redirect()->toRoute('secure');
		}

        $this->getEvent()->getViewModel()->setVariables(
            array(
                'is_logged_in'           => $isLoggedIn,
                'noBetaFeedbackLink'     => TRUE
            )
        );

        return array();
    }

    /**
     * Renders the register form and processes the posted data
     *
     * @return array(Registration for instance).
     */
	public function activateAction()
	{   
        $activationCode = $this->params()->fromRoute('activationid', 0);

        if ($activationCode) {

            if ($this->getUsersTable()->activateUser($activationCode)) {

                /* if the user is successfully activated then collect his/her 
                 user id and email id.*/
                $userRecord = $this
                                ->getUsersTable()
                                ->getUserDetailByActivationCode(
                                        $activationCode
                );

                if ($userRecord) {

                    $this
                        ->getUsersPermTempTable()
                        ->copyPermissions(
                                $userRecord['email_login'], 
                                $userRecord['userid']
                    );
                }

                $this
                    ->flashmessenger()
                    ->addMessage("Successfully activated the account related to " 
                            . $userRecord['email_login']
                );
            }
            else {
                $this
                    ->flashmessenger()
                    ->addMessage("Your login has been validated.");
            }

            return $this->redirect()->toRoute('login');
        }
    }

	/**
     * Apply strip tags to the provided input
     * 
     * @param String $valueToFilter
     * @return Filtered string on Success and false on failure.
     */
	public function filterFormElements($valueToFilter = '')
	{
		// instantiate the strip tag filter
		$stripTagFilter   = new StripTags();
		$StringTrimFilter = new StringTrim();

		if ($valueToFilter) {

			return $stripTagFilter
                        ->filter(
                                $StringTrimFilter->filter($valueToFilter)
            );
		}
		return false;
	}

    /**
     * Check whether both the entered password and confirm password are matched.
     * 
     * @param String $password
     * @param String $confirmPassword
     * @return boolean true on success and false on failure.
     */
	public function validateConfirmPassword(
            $password = '', 
            $confirmPassword = ''
    )
	{
		if ($password && $confirmPassword) {

			if ($password === $confirmPassword) {

				return true;
			}
		}
		return false;
	}

    /**
     * Function to render terms of service without any layout.
     * 
     */
    public function termsOfServiceAction()
    {
        $config = $this->getServiceLocator()->get('Config');
        
        $result = new ViewModel(
            array(
                'baseUrl'  => $config['base_url']
            )
        );
        $result->setTerminal(true);
        return $result;
    }

    /**
     * Function to render terms of service without any layout.
     * 
     */
    public function privacyPolicyAction()
    {
        $config = $this->getServiceLocator()->get('Config');
        
        $result = new ViewModel(
            array(
                'baseUrl'  => $config['base_url']
            )
        );
        $result->setTerminal(true);
        return $result;
    }

    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersTable()
	{
		// check if usersTable previously defined.
		if (! $this->usersTable) {

			$sm = $this->getServiceLocator();
			$this->usersTable = $sm->get('Secure\Model\UsersTable');
		}
		return $this->usersTable;
	}
    
    /**
     * Function to create an encoded string from the array provided as input.
     * 
     * @param Array $inputArray
     * @return String encoded string on success and false on failure.
     */
	public function createEncodedString($inputArray)
    {
        if (is_array($inputArray)) {
            $convert = $this->getServiceLocator()->get('ConvertParams');
            $string = $convert->arrayToString($inputArray);
            return $convert->paramEncrypt($string);
        }
        return false;
    }

    /**
     * Get an instance of usersTable by using service manager.
     *
     * @return service manager instance for users table.
     */
	public function getUsersPermTempTable()
	{
		// check if usersTable previously defined.

		if (! $this->usersPermTempTable) {

			$sm = $this->getServiceLocator();
			$this->usersPermTempTable = $sm 
                                        ->get('Secure\Model\UsersPermTempTable');
		}
		return $this->usersPermTempTable;
	}
    
    /**
     * Get an instance of requestTrackerTable by using service manager.
     *
     * @return service manager instance for request_tracker table.
     */
	public function getRequestTrackerTable()
	{
		// check if usersTable previously defined.
		if (! $this->requestTrackerTable) {
			$sm = $this->getServiceLocator();
			$this->requestTrackerTable = $sm->get('Secure\Model\RequestTrackerTable');
		}
		return $this->requestTrackerTable;
	}
}

?>