<?php

//module/Secure/src/Secure/Model/CalendarTable.php

/**************
	Purpose: Model functions for accessing the calendar table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Secure\Model\Events;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert, Zend\Db\Sql\Where, Zend\Db\Sql\Predicate;
use Zend\Db\Sql\Select;
use Zend\Db\Adapter\Adapter;

class CalendarTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
    
    /**
     * @var dbSql
     */
	protected $dbSql;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;

	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
	
    /**
     * Function to get the calendarid from the input resource id.
     * 
     * @param int $resourceid
     * @return boolean on success return the required record and false on failure.
     * 
     */
	public function getCalendaridOfResource($resourceid)
    {
		
		$rowset = $this->tableGateway->select(array('fk_resourceid' => $resourceid));
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Function to get the calendar name from the given calendar id.
     * 
     * @param int $calendar
     * @return boolean on success return the required record and false on failure.
     * 
     */
	public function getCalendarNameFromId($calendarid)
    {
		
		$rowset = $this->tableGateway->select(array('calendarid' => $calendarid));
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Function to update the calendar name using the given calendar id.
     * 
     * @param int $calendarid
     * @param string $calendar_name
     * @return boolean true if successfully updated and false on failure.
     * 
     */
	public function updateCalendarRecordById($calendarid, $calendar_name)
    {
        $updataUsers = array(
            'calendar_name' =>  $calendar_name
        );
        
        $updateResource = array(
			'description'           => $calendar_name,
            'updated'  				=>  date('Y-m-d H:i:s')
		);
        
        try {
            $this->tableGateway->update($updataUsers, array('calendarid' => $calendarid));
            
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $calendarid,
                            'type'          => 'calendar'
                        )
            );
            return true;
        } catch (Exception $exc) {
            return false;
        }
        // system has undergone an unknown exception.
    }
    
    /**
     * Creates a calendar resource
     * 
     * @param string $calendarName
     * @param int $calendarParentResourceId
     * @param int $userId
     * @return boolean true if successfully created calendar resource false if faiure.
     * 
     */
	public function createCalendarResource(
            $calendarName, 
            $calendarParentResourceId, 
            $userId
    )
    {
        try {
            $calendarData = array(
                'calendar_name'			=>	$calendarName,
                'created'               =>  date('Y-m-d H:i:s'),
                'userid_created'        =>  $userId
            );
            $this->tableGateway->insert($calendarData);
            $lastInsertIdCalendar = $this->tableGateway->lastInsertValue;
            $resourceData = array(
				'type'					=>	'calendar',
				'fk_type_recid'			=> 	$lastInsertIdCalendar,
                'description'           =>  'Calendar',
				'fk_userid_owner'  		=> 	$userId,
				'fk_resourceid_parent' 	=>  $calendarParentResourceId,
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $userId,
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->insert($resourceData);
            return true;
        }
        catch (\Exception $e) {
            return false;
        }
    }
}