<?php
//module/Secure/src/Secure/Model/Editusers.php

/**************
	Purpose: Model functions for editing the users.
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;

use Zend\InputFilter\Factory as InputFactory;     // <-- Add this import
use Zend\InputFilter\InputFilter;                 // <-- Add this import
use Zend\InputFilter\InputFilterAwareInterface;   // <-- Add this import
use Zend\InputFilter\InputFilterInterface;        // <-- Add this import
use Zend\Validator\EmailAddress;

class Editusers implements InputFilterAwareInterface
{
	/**
     * @var id
     */
    public $user_id;
	
	/**
     * @var user_name
     */
    public $first_name;
	
	/**
     * @var user_name
     */
    public $last_name;
	
	/**
     * @var pass_word
     */
    public $pass_word;
	
	/**
     * @var email_address
     */
    public $email_address;

	/**
     * Preparing the posted data to store in database.
     *
     * @param  $data(array) with arbitrary key values.
     */
    public function exchangeArray($data)
    {
		
        $this->first_name 
                = (isset($data['first_name'])) ? $data['first_name'] : null;
        $this->last_name 
                = (isset($data['last_name'])) ? $data['last_name'] : null;
        $this->pass_word = (isset($data['pass_word'])) ? $data['pass_word'] : null;
        $this->email_address 
                = (isset($data['email_address'])) ? $data['email_address'] : null;
    }
	
	/**
     * Sets input filter.
     *
     * @param  InputFilterInterface $inputFilter
     * @throws \Exception
     */
    public function setInputFilter(InputFilterInterface $inputFilter)
    {
        throw new \Exception("Not used");
    }

	/**
     * Retrieve input filter. Associate validation to the input fields in register form.
	 * @return \Zend\InputFilter\InputFilterInterface
     */
    public function getInputFilter()
    {
		// check if inputFilter field is initialized
        if (!$this->inputFilter) {
            $inputFilter = new InputFilter();
            $factory     = new InputFactory();
			
			// add filter and validators to username field
            $inputFilter->add($factory->createInput(array(
                'name'     => 'first_name',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY 
                                                            => 'Please enter First name.' 
                            ),
                        ),
                    ),
                    array(
                        'name' => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min' => 0,
                            'max' => 100,
                            'messages' => array(
                                'stringLengthTooLong' 
                                => 'The entered user name must be less than 100 characters!' 
                            ),
                        ),
                    ),
                ),
            )));
			
			// add filter and validators to username field
            $inputFilter->add($factory->createInput(array(
                'name'     => 'last_name',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY 
                                                            => 'Please enter Last name.' 
                            ),
                        ),
                    ),
                    array(
                        'name' => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min' => 0,
                            'max' => 100,
                            'messages' => array(
                                'stringLengthTooLong' 
                                => 'The entered user name must be less than 100 characters!' 
                            ),
                        ),
                    ),
                ),
            )));

			// add filter and validators to password field
            $inputFilter->add($factory->createInput(array(
                'name'     => 'pass_word',
                'required' => false,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(	
                    array(
                        'name' => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min' => 1,
                            'max' => 100,
                            'messages' => array(
                                'stringLengthTooLong' 
                                => 'The entered Password must be less than 100 characters!'
                            ),
                        ),
                    ),
                ),
            )));
			
			// add filter and validators to confirm password field
			$inputFilter->add($factory->createInput(array(
                'name'     => 'conf_password',
                'required' => false,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                        'name' => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min' => 1,
                            'max' => 100,
                            'messages' => array(
                                'stringLengthTooLong' => 'The entered Confirm password 
                                    must be less than 100 characters!' 
                            ),
                        ),
                    )
                ),
            )));
			
			// add filter and validators to email address field.
			$inputFilter->add($factory->createInput(array(
                'name'     => 'email_address',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
					array(
                      'name' =>'NotEmpty', 
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\NotEmpty::IS_EMPTY 
                                    => 'Please enter E-mail address.' 
                            ),
                        ),
                    ),
					array(
                      'name' =>'EmailAddress',
                        'options' => array(
                            'messages' => array(
                                \Zend\Validator\EmailAddress::INVALID 
                                    => 'Please enter E-mail address on format abc@abc.com.' 
                            ),
                        ),
                    ),
                ),
            )));

            $this->inputFilter = $inputFilter;
        }

        return $this->inputFilter;
    }
}