<?php

//module/Secure/src/Secure/Model/MedicationTable.php

/**************
	Purpose: Model functions for accessing the Medication table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert;
use Zend\Db\Adapter\Adapter;

class MedicationTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	private $sessionInstance;
	
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;
    
	/**
     * @var form
     */
	protected $eventsTable;
	protected $dbSql;
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }

	/**
     * Function to save a medication record.
     * 
     * @param array $insertMedicationData
     * @param int $userid
     * @return boolean  true on success and false on failure.
     * 
     */
    public function saveMedication($insertMedicationData, $userid)
    {
        $medicationData = array(
            'med_name'          => $insertMedicationData['med_name'],
            'generic_name'      => $insertMedicationData['generic_name'],
            'dosage'            => $insertMedicationData['dosage'],
            'dosage_frequency'  => $insertMedicationData['dosage_frequency'],
            'med_notes'         => $insertMedicationData['med_notes'],
            'physician'         => $insertMedicationData['physician'],
            'date_prescribed'   => $insertMedicationData['date_prescribed'],
            'pharmacy'          => $insertMedicationData['pharmacy'],
            'symptoms_address'  => $insertMedicationData['symptoms_address'],
            'side_effects'      => $insertMedicationData['side_effects'],
            'drug_interactions' => $insertMedicationData['drug_interactions'],
            'physical_desc'     => $insertMedicationData['physical_desc'],
            'discont_medication' => $insertMedicationData['discont_medication'],
            'created'           => date('Y-m-d H:i:s'),
            'userid_created'    => $userid
        );
		try {
            
			$this->tableGateway->insert($medicationData);
			$lastInsertIdPerson = $this->tableGateway->lastInsertValue;
			$resourceData = array(
				'type'					=>	'medication',
				'fk_type_recid'			=> 	$lastInsertIdPerson,
                'description'           =>  $insertMedicationData['med_name'],
				'fk_userid_owner'  		=> 	$userid,
				'fk_resourceid_parent' 	=>  $insertMedicationData['parent_resource_id'],
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $userid,
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->insert($resourceData);
            return true;
		}
		catch (\Exception $e) {
			return false;
		}
    }
    
    /**
     * Model function to update a medication record.
     * 
     * @param array $updateMedicationData
     * @param int $userid
     * @return boolean  ture on success and false on failure.
     * 
     */
    public function updateMedication($updateMedicationData, $userid)
    {
        $medicationData = array(
            'med_name'          => $updateMedicationData['med_name'],
            'generic_name'      => $updateMedicationData['generic_name'],
            'dosage'            => $updateMedicationData['dosage'],
            'dosage_frequency'  => $updateMedicationData['dosage_frequency'],
            'med_notes'         => $updateMedicationData['med_notes'],
            'physician'         => $updateMedicationData['physician'],
            'date_prescribed'   => $updateMedicationData['date_prescribed'],
            'pharmacy'          => $updateMedicationData['pharmacy'],
            'symptoms_address'  => $updateMedicationData['symptoms_address'],
            'side_effects'      => $updateMedicationData['side_effects'],
            'drug_interactions' => $updateMedicationData['drug_interactions'],
            'physical_desc'     => $updateMedicationData['physical_desc'],
            'discont_medication' => $updateMedicationData['discont_medication'],
            'userid_updated'    => $userid
        );
        
        $updateResource = array(
            'description'           =>  $updateMedicationData['med_name'], 
            'updated'  				=>  date('Y-m-d H:i:s')
        );
        
		try {
			$this->tableGateway->update(
                    $medicationData, 
                    array('medicationid' => $updateMedicationData['medicationid'])
            );
            
			// also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                        $updateResource,
                        array(
                            'fk_type_recid' => $updateMedicationData['medicationid'],
                            'type'          => 'medication'
                        )
            );
            return true;
        }
        catch (\Exception $e) {
			return false;
		}
    }
    
    /**
     * Model function to get all medications by resource id.
     * 
     * @param type $resourceid
     * @return boolean
     * 
     */
    public function getAllMedications($resourceid)
    {
        if ($resourceid) {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT *
                    FROM resource AS t1
                    LEFT JOIN medication AS t2
                    ON ( t1.fk_type_recid = t2.medicationid)
                    WHERE t1.fk_resourceid_parent = $resourceid  
                    AND t1.type='medication' AND t1.rec_status = 1",
                    Adapter::QUERY_MODE_EXECUTE);

            if ($resultSet) {
                return $resultSet;
            }
        }
        return false;
    }
    
    /**
     * Model function to get a medication record by id.
     * 
     * @param type $medicationId
     * @return boolean  (object) on success and false on failure.
     * 
     */
    public function getMedicationRecordById($medicationId)
    {
        $rowset = $this
                    ->tableGateway
                    ->select(
                            array('medicationid' => $medicationId)
        );

        $row = $rowset->current();

        if ($row) {
           return $row;
        }
        return false;
    }
    
    /**
     * Delete medication
     * 
     * @param type $userResourceId
     * @return boolean true on success and false on failure
     */
    public function deleteMedication($medicationId)
    {
        
        if ($medicationId) {
             if ($this->tableGateway->delete(array('medicationid' => $medicationId))) {
                 $resourceInstance = new TableGateway('resource', $this->dbSql);
                 if ($resourceInstance->delete(
                         array('fk_type_recid' => $medicationId, 'type'=>'medication'))) {
                     return true;
                 }
             }
        }
        return false;
    }
}