<?php

//module/Secure/src/Secure/Model/PersonTable.php

/**************
	Purpose: Model functions for accessing the person table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert;

class PersonTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	private $sessionInstance;
	
	/**
     * @var form
     */
	protected $eventsTable;
	protected $dbSql;
    
    /*
     * Defined constants for permission
     */
    const IS_OWNER    = 1;
    const CAN_EDIT    = 2;
    const CAN_COMMENT = 3;
    const CAN_VIEW    = 4;
    
    /*
     * Defined constants for various Resource types
     */
    const RESOURCE_TABLE            = 'resource';
    const RESOURCE_TYPE_TABLE       = 'resource_type';
    const FOLDER_TABLE              = 'folders';
    
    const FOLDER_RESOURCE           = 'folder';
    const FOLDER_DESC               = 'Profiles';
    
    const PERSON_RESOURCE           = 'person';
    
    const HEALTH_AND_MEDICAL_RESOURCE   = 'health&medical';
    const HEALTH_AND_MEDICAL_DESC       = 'Health and Medical';
    
    const FINANCIAL_RESOURCE            = 'financial';
    const FINANCIAL_DESC                = 'Financial';
    
    const LEGAL_RESOURCE                = 'legal';
    const LEGAL_DESC                    = 'Legal';
    
    const POST_RESOURCE                 = 'post';
    const EVENT_RESOURCE                = 'event';
    const TASK_RESOURCE                 = 'task';
    const DOCUMENT_RESOURCE             = 'document';
    const CONTACT_RESOURCE              = 'contact';
    const MEDICATION_RESOURCE           = 'medication';
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }

    /**
     * Model function to save a person record.
     * 
     * @param array $user
     * @param int $root_folder_id
     * @return boolean true on success and false on failure.
     */
    public function savePerson($user, $root_folder_id)
    {
		// prepare data to insert into the database
		$insertPersonData = array(
			'fk_userid'         => 0,
			'first_name'        => $user['firstName'],
			'last_name'         => $user['lastName'],
			'phone_main'        => $user['phoneMain'],
			'phone_mobile'      => $user['phoneMobile'],
			'phone_emergency'	=> $user['phoneEmergency'],
			'address1'          => $user['address1'],
			'address2'          => $user['address2'],
			'city'              => $user['city'],
			'state'             => $user['state'],
			'zipcode'           => $user['zipcode'],
			'email_primary'     => $user['emailPrimary'],
			'email_secondary'   => $user['emailSecondary'],
			'notes'             => $user['notes'],
			'created'           => date('Y-m-d H:i:s'),
			'userid_created'    => $user['userId']
		);
        
        $resourceInstance     = new TableGateway(self :: RESOURCE_TABLE, $this->dbSql);
        $resourceTypeInstance = new TableGateway(self :: RESOURCE_TYPE_TABLE, $this->dbSql);
		
        $connection = $this->dbSql->getDriver()->getConnection();
        
		try {
            $connection->beginTransaction();
			$this->tableGateway->insert($insertPersonData);
			$lastInsertIdPerson = $this->tableGateway->lastInsertValue;
			$resourceData = array(
				'type'					=>	self :: PERSON_RESOURCE,
                'description'           =>  $user['firstName'] . ' ' . $user['lastName'],
				'fk_type_recid'			=> 	$lastInsertIdPerson,
				'fk_userid_owner'  		=> 	$user['userId'],
				'fk_resourceid_parent' 	=>  $root_folder_id,
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $user['userId'],
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance->insert($resourceData);
            $lastInsertIdResource = $resourceInstance->lastInsertValue;
            
            $resourceTypeData = array(
                'resourceid'        =>  $lastInsertIdResource,
                'type'              =>  self :: POST_RESOURCE,
                'sort_order'        =>  '1',
                'created'           =>  date('Y-m-d H:i:s'),
				'userid_created'    =>  $user['userId'],
                'updated'  			=>  date('Y-m-d H:i:s')
			);
            
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type']       = self :: EVENT_RESOURCE;
            $resourceTypeData['sort_order'] = '2';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type'] = self :: TASK_RESOURCE;
            $resourceTypeData['sort_order'] = '3';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type'] = self :: DOCUMENT_RESOURCE;
            $resourceTypeData['sort_order'] = '4';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type'] = self :: CONTACT_RESOURCE;
            $resourceTypeData['sort_order'] = '5';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceDataHealth = array(
				'type'					=>	self :: FOLDER_RESOURCE,
                'description'           =>  self :: HEALTH_AND_MEDICAL_DESC,
				'fk_userid_owner'  		=> 	$user['userId'],
				'fk_resourceid_parent' 	=>  $lastInsertIdResource,
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $user['userId'],
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance->insert($resourceDataHealth);
            
            $resourceTypeData['resourceid'] = $resourceInstance->lastInsertValue;
            
            $resourceTypeData['type'] = self :: MEDICATION_RESOURCE;
            $resourceTypeData['sort_order'] = '1';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceTypeData['type'] = self :: DOCUMENT_RESOURCE;
            $resourceTypeData['sort_order'] = '2';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceDataFinancial = array(
				'type'					=>	self :: FOLDER_RESOURCE,
                'description'           =>  self :: FINANCIAL_DESC,
				'fk_userid_owner'  		=> 	$user['userId'],
				'fk_resourceid_parent' 	=>  $lastInsertIdResource,
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $user['userId'],
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance->insert($resourceDataFinancial);
            $resourceTypeData['resourceid'] = $resourceInstance->lastInsertValue;
            $resourceTypeData['type'] = self :: DOCUMENT_RESOURCE;
            $resourceTypeData['sort_order'] = '1';
            $resourceTypeInstance->insert($resourceTypeData);
            
            $resourceDataLegal = array(
				'type'					=>	self :: FOLDER_RESOURCE,
                'description'           =>  self :: LEGAL_DESC,
				'fk_userid_owner'  		=> 	$user['userId'],
				'fk_resourceid_parent' 	=>  $lastInsertIdResource,
				'created'  				=>  date('Y-m-d H:i:s'),
				'userid_created' 		=>  $user['userId'],
                'updated'  				=>  date('Y-m-d H:i:s')
			);
			
			$resourceInstance->insert($resourceDataLegal);
            $resourceTypeData['resourceid'] = $resourceInstance->lastInsertValue;
            $resourceTypeData['type'] = self :: DOCUMENT_RESOURCE;
            $resourceTypeData['sort_order'] = '1';
            $resourceTypeInstance->insert($resourceTypeData);
			$connection->commit();
			return true;
		}
		catch (\Exception $e) {
            $connection->rollback();
			return false;
		}
    }
	
	/**
     * Model function to get the user name by user id.
     * 
     * @param int $user_id
     * @return boolean (object) on success and false on failure.
     * 
     */
    function getUserName($user_id) {
	
		$id  = (int) $user_id;
        $rowset = $this->tableGateway->select(array('fk_userid' => $id, 'rec_status' => 1));
        $row = $rowset->current();
		return $row;
        if (!$row) {
            return false;
        }
        return $row;
	}
	
    /**
     * Model function to get person information by person id.
     * 
     * @param int $personId
     * @return boolean (object) on success and false on failure.
     * 
     */
	function getPersonInfo($personId) {
	
		$id  = (int) $personId;
        $rowset = $this->tableGateway->select(array('personid' => $id, 'rec_status' => 1));
        $row = $rowset->current();
		return $row;
        if (!$row) {
            return false;
        }
        return $row;
	}
	
    /**
     * Model function to update a person record.
     * 
     * @param array $updataPerson
     * @param int $personId
     * 
     * @return boolean (object) on success and false on failure.
     */
	function updatePersonInfo($updataPerson, $personId) {
	
		$id  = (int) $personId;
        
        $updateResource = array(
			'description'   => $updataPerson['first_name'] . ' ' . $updataPerson['last_name'],
            'updated'  		=>  date('Y-m-d H:i:s')
		);
		
		try {
			$this->tableGateway->update($updataPerson, array('personid' => $id));
            
            // also update the description field in the resource table.
            $resourceInstance = new TableGateway('resource', $this->dbSql);
			$resourceInstance->update(
                                $updateResource,
                                array(
                                    'fk_type_recid' => $personId,
                                    'type'          => 'person'
                                )
                    );
		}
		catch (\Exception $e) {
			return false;
		}
		return true;
	}
}