<?php

//module/Secure/src/Secure/Model/ResourceTable.php

/**************
	Purpose: Model functions for accessing the resource table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Zend\Session\Container;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert, Zend\Db\Sql\Where, Zend\Db\Sql\Predicate;
use Zend\Db\Sql\Select;
use Zend\Db\Adapter\Adapter;

class ResourceTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
	
	/**
     * @var sessionInstance
     */
	private $sessionInstance;
	
	/**
     * @var eventsTable
     */
	protected $eventsTable;
	
	/**
     * @var dbSql
     */
	protected $dbSql;
    
	/**
     * @var table
     */
	protected $table = 'resource';
	
	/**
     * Initialize sessionInstance variable with session instance.
     *
     * @return (object)the intialzed $sessionInstance.
     */
	public function intializeSessionInstance()
    {
		$this->sessionInstance = new Container('containername');
		return $this->sessionInstance;
	}
	
	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Get all resources that are being linked with the given userid.
     * 
     * @param int $userid
     * 
     */
    public function getAllResourcesUnderRoot($userId)
    {
        if ($userId) {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT DISTINCT(t2.resourceid), t2.description, t2.type, 
                    t2.updated AS updated, t2.fk_type_recid AS pid, t1.perm_type AS perm_type,
                    t2.fk_userid_owner AS ownerid, t4.first_name AS owner_firstname,
                    t4.last_name AS owner_lastname, t3.fk_userid AS person_owner_id
                    FROM user_resource AS t1
                    RIGHT OUTER JOIN `resource` AS t2
                    ON (t2.resourceid = t1.fk_resourceid AND t1.fk_userid = $userId)
                    LEFT JOIN person AS t3 
                    ON ( t2.fk_type_recid = t3.personid AND type = 'person' )
                    LEFT OUTER JOIN person  AS t4
                    ON (t2.fk_userid_owner = t4.fk_userid)
                    WHERE t2.fk_userid_owner = $userId
                    OR t1.fk_userid = $userId
					ORDER BY IF( t3.fk_userid = $userId, 1, 0 ) DESC, t2.description ASC ",
                    Adapter::QUERY_MODE_EXECUTE);
            
            if ($resultSet) {
                return $resultSet;
            }
        }
        return false;
    }
    
    /**
     * Get all records from the resource table and list them in the left menu.
     * 
     * @param int $rootFolderId
     * @return boolean
     * 
     */
    public function getResourcesUnderRootFolder($rootFolderId)
    {
        if ($rootFolderId) {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT t1.resourceid, t2.first_name, t2.last_name, 
                    t3.provider_name, t1.type, t4.first_name AS owner_firstname,
                    t2.updated AS person_updated, t3.updated AS provider_updated,
                    t1.fk_type_recid AS pid, t4.last_name AS owner_lastname,
                    FROM resource AS t1
                    LEFT JOIN person AS t2
                    ON ( t1.fk_type_recid = t2.personid AND t1.type = 'person' )
                    LEFT JOIN provider AS t3
                    ON ( t1.fk_type_recid = t3.providerid AND t1.type = 'provider' )
                    LEFT JOIN person AS t4
                    ON ( t1.fk_userid_owner = t4.fk_userid )
                    WHERE t1.fk_resourceid_parent = $rootFolderId 
                    AND t1.rec_status = 1",
                    Adapter::QUERY_MODE_EXECUTE);

            if ($resultSet) {
                return $resultSet;
            }
        }
        return false;
    }
    
    /**
     * Get all records from the resource table and present under given resource id.
     * 
     * @param int $resourceid
     * @return boolean (object) on success and false on failure.
     * 
     */
    public function getResourcesUnderContainer($resourceid, $userId)
    {
        if ($resourceid) {
            $sql = $this->dbSql;
			
            $resultSet = $sql->query("SELECT t1.resourceid, t1.type, 
                    t1.description AS description, t2.first_name AS owner_firstname,
                    t1.fk_userid_owner AS ownerid,
                    t1.updated AS updated, t1.fk_type_recid AS pid, t2.last_name AS owner_lastname
                    FROM resource AS t1
                    LEFT JOIN person AS t2
                    ON ( t1.fk_userid_owner = t2.fk_userid )
                    WHERE t1.fk_resourceid_parent = $resourceid AND t1.type = 'folder'
                    AND t1.rec_status = 1",
                    Adapter::QUERY_MODE_EXECUTE);
			
            if ($resultSet) {
                return $resultSet;
            }
        }
        return false;
    }
    
	/**
     * Model function to list all resources for which the given resource id is
     * owner.
     * 
     * @param type $userId
     * @return boolean (object) on success and false on failure.
     */
    public function listResources($userId)
    {
		$sql = $this->dbSql;
		$resultSet = $sql->query("SELECT t1.type, t1.resourceid, t1.fk_resourceid_parent,
                t2.updated AS person_updated,
                t3.updated AS provider_updated,
                t1.fk_type_recid, t1.fk_userid_owner, t2.first_name, 
                t2.last_name, t3.provider_name, t4.first_name AS firstname,
                t4.last_name AS lastname
                FROM resource AS t1
                LEFT JOIN person AS t2 
                ON ( t1.fk_type_recid = t2.personid AND TYPE = 'person' )
                LEFT JOIN provider AS t3 
                ON ( t1.fk_type_recid = t3.providerid AND TYPE = 'provider' )
                LEFT JOIN person AS t4 
                ON ( t1.fk_userid_owner = t4.fk_userid )
                WHERE t1.rec_status = 1
                AND t1.fk_userid_owner = $userId 
                AND ( t1.type = 'person' OR t1.type = 'provider' )",
				Adapter::QUERY_MODE_EXECUTE);

		if ($resultSet) {
			return $resultSet;
		}
		return false;
    }
    
    /**
     * List of all the resource associated with folders.
     * 
     * @param int $user_id
     * @return boolean (object) on success and false on failure.
     */
    public function listResourcesByFolders($userId)
    {
        $sql = $this->dbSql;
		$resultSet = $sql->query("SELECT t1.type AS rec_type, t2.first_name AS firstname, 
                t2.last_name AS lastname,
                t3.provider_name AS providername, t4.calendar_name AS calendarname,
                t5.folder_name AS foldername
                FROM resource AS t1
                LEFT JOIN person AS t2 
                ON ( t1.fk_type_recid = t2.personid AND t1.type = 'person' )
                LEFT JOIN provider AS t3
                ON ( t1.fk_type_recid = t3.providerid AND t1.type = 'provider' )
                LEFT JOIN calendar AS t4 
                ON ( t1.fk_type_recid = t4.calendarid AND t1.type = 'calendar' )
                LEFT JOIN folders AS t5
                ON ( t1.fk_type_recid = t5.folderid AND t1.type = 'folder')
                WHERE t1.rec_status =1
                AND t1.fk_userid_owner = $userId",
				Adapter::QUERY_MODE_EXECUTE);

		if ($resultSet) {
			return $resultSet;
		}
		return false;
    }
	
	/**
     * Model function to get the container name by resourceid.
     * 
     * @param int $resourceid
     * @return boolean (string) i.e. container name on success and false on failure.
     */
    public function getContainerName($resourceid)
    {
        $rowset = $this->tableGateway->select(array('resourceid' => $resourceid));
        $row = $rowset->current();

        if ($row->type == 'person') {
            $personInstance = new TableGateway('person', $this->dbSql);
            $personset = $personInstance->select(
                    array('personid' => $row->fk_type_recid)
            );
            $row = $personset->current();
        }
        else if ($row->type == 'provider') {
            $providerInstance = new TableGateway('provider', $this->dbSql);
            $personset = $providerInstance->select(
                    array('providerid' => $row->fk_type_recid)
            );
            $row = $personset->current();
        }

        if (!$row) {
            return false;
        }
        if (!$row->provider_name) {
            return $row->first_name . ' ' . $row->last_name;
        }
        else {
            return $row->provider_name;
        }
    }   
     
     /**
      * Get all calendar records for the given resource id(provider/cared-for)
      * 
      * @param int $resourceid
      * @return boolean
      */
     public function getAllCalendarRecords($resourceid)
     {
        try {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT *
                    FROM resource AS t1
                    LEFT JOIN calendar AS t2
                    ON ( t1.fk_type_recid = t2.calendarid AND TYPE = 'calendar' )
                    WHERE t1.rec_status = 1
                    AND t1.fk_resourceid_parent = $resourceid",
                    Adapter::QUERY_MODE_EXECUTE);

            if ($resultSet) {
                return $resultSet;
            }
        }
        catch (\Exception $e) {
            return false;
        }
        
        // system has undergone an unknown exception
     }
     
     /**
      * Model function to get all event records using the given resourceid.
      * 
      * @param int $resourceid
      * @return boolean
      */
     public function getAllEventRecords($resourceid)
     {
        try {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT *
                    FROM resource AS t1
                    LEFT JOIN events AS t2
                    ON ( t1.fk_type_recid = t2.eventid AND TYPE = 'event' )
                    WHERE t1.rec_status = 1
                    AND t1.fk_resourceid_parent = $resourceid",
                    Adapter::QUERY_MODE_EXECUTE);

            if ($resultSet) {
                return $resultSet;
            }
        }
        catch (\Exception $e) {
            return false;
        }
        
        // system has undergone an unknown exception
     }
     
     /**
      * Get parent resource record from the given resourceid.
      * 
      * @param int $resourceid
      * @return array if record exist and false if not.
      */
     public function getParentResourceRecord($resourceid)
     {
         // first get the parent resource id of the given resource.
         $rowset = $this
                    ->tableGateway
                    ->select(
                            array(
                                    'resourceid'  => $resourceid
                            )
        );
        $row = $rowset->current();
        
        if ($row) {
            
            //with the help of parent resource id get the record details
            $rowset = $this
                       ->tableGateway
                       ->select(
                               array(
                                       'resourceid'  => $row['fk_resourceid_parent']
                                   )
           );
           $row = $rowset->current();
           if ($row) {
               return $row;
           }
        }
        return false;
     }
     
     /**
      * Get parent resource id by type_recid and type
      * 
      * @param int $typeRecId
      * @param string $type
      * @return boolean (object) on success and false on failure.
      */
     public function getParentResourceIdByType($typeRecId, $type)
     {
        //with the help of parent resource id get the record details
        $rowset = $this
                  ->tableGateway
                  ->select(
                    array(
                              'fk_type_recid'   => $typeRecId,
                              'type'            => $type
                    )
        );
        $row = $rowset->current();
        if ($row) {
          return $row;
        }
        return false;
     }
     
     /**
      * Get resource type by id.
      * 
      * @param type $resourceid
      * @return array if record exist and false if not.
      * 
      */
     public function getResourceTypeById($resourceid) {
         
         $rowset = $this
                  ->tableGateway
                  ->select(
                          array(
                                  'resourceid'  => $resourceid
                              )
        );
        $row = $rowset->current();
        if ($row) {
          return $row;
        }
        return false;
     }
     
     /**
      * Create breadcrumb by taking the resource id into account.
      * 
      * @param type $resourceid
      * @return string|boolean
      */
     public function createBreadcrumb($resourceid, $userId)
     {
         $breadCrumbString = '';
         if ($resourceid) {
            
            $rowset = $this
                     ->tableGateway
                     ->select(
                            array(
                                    'resourceid'  => $resourceid
                            )
           );

           $row = $rowset->current();
           
           // created the encrypted resource id string.
           $encodeArray = array('resourceid' => $resourceid);
           $breadCrumbString = $this->arrayToString($encodeArray);
           
           $breadCrumbEncodedString1 = $this->paramEncrypt($breadCrumbString);

           if ($row['type'] == 'person' || $row['type'] == 'provider') {
               
               if ($row['type'] == 'person') {
                    // collect the person record using type_recid.
                    $personInstance = new TableGateway('person', $this->dbSql);
                    $personRowset = $personInstance->select(
                            array(
                                    'personid'  => $row['fk_type_recid'],
                                    'rec_status'=> 1
                            )
                    );
                    $personRow = $personRowset->current();
                    
                    if ($personRow['fk_userid'] == $userId) {
                        
                        $breadCrumbString = '<a href="/secure//' 
                                . $breadCrumbEncodedString1 
                                . '">My Profile </a>&rsaquo;&rsaquo; ';
                    }
                    else {
                        $breadCrumbString = '<a href="/secure//' 
                                . $breadCrumbEncodedString1 . '">' 
                                . $row['description'] . '</a> &rsaquo;&rsaquo; ';
                    }
               }
               else {
                    $breadCrumbString = '<a href="/secure//' . $breadCrumbEncodedString1 . '">' 
                            . $row['description'] . '</a> &rsaquo;&rsaquo; ';
               }
           }
           else {
               $breadCrumbString =  '<a href="/secure//' . $breadCrumbEncodedString1 . '">' 
                       . $row['description'] . '</a>' ;

               $rowset = $this
                           ->tableGateway
                           ->select(
                                   array(
                                           'resourceid'  => $row['fk_resourceid_parent']
                                   )
               );

               $row = $rowset->current();
               
               // created the encrypted resource id string.
               $encodeArray = array('resourceid' => $row['resourceid']);
               $breadCrumbString1 = $this->arrayToString($encodeArray);

               $breadCrumbEncodedString2 = $this->paramEncrypt($breadCrumbString1);
               
               if ($row['type'] == 'person' || $row['type'] == 'provider') {
                   
                   if ($row['type'] == 'person') {
                        // collect the person record using type_recid.
                        $personInstance = new TableGateway('person', $this->dbSql);
                        $personRowset = $personInstance->select(
                                array(
                                        'personid'  => $row['fk_type_recid'],
                                        'rec_status'=> 1
                                )
                        );
                        $personRow = $personRowset->current();

                        if ($personRow['fk_userid'] == $userId) {
                            $breadCrumbString = '<a href="/secure//' . $breadCrumbEncodedString2 . '">' 
                                    . 'My Profile</a> &rsaquo;&rsaquo; ' . $breadCrumbString;
                        }
                        else {
                            $breadCrumbString = '<a href="/secure//' . $breadCrumbEncodedString2 
                                    . '">' . $row['description'] . '</a> &rsaquo;&rsaquo; ' 
                                    . $breadCrumbString;
                        }
                   }
                   else {
                        $breadCrumbString = '<a href="/secure//' . $breadCrumbEncodedString2 
                                    . '">' . $row['description'] . '</a> &rsaquo;&rsaquo; ' 
                                    . $breadCrumbString;
                   }
               }
            }
            return '<a href="/">Home</a> &rsaquo;&rsaquo; ' . $breadCrumbString;
         }
         return false;
     }
     
     /**
     * Convert a Ascii to hexadecimal
     * 
     * @param string $ascii
     * @return string $hex
     */
	function AsciiToHex($ascii) {
		$hex = '';

		for($i = 0; $i < strlen($ascii); $i++)
		$hex .= str_pad(base_convert(ord($ascii[$i]), 10, 16), 2, '0', STR_PAD_LEFT);

		return $hex;
	}

	/**
     * Convert a hex string to ascii, prepend with '0' if input is not an even number
     * of characters in length   
     * 
     * @param string $hex
     * @return string $ascii
     */
	function HexToAscii($hex) {
		$ascii = '';

		if (strlen($hex) % 2 == 1)
		$hex = '0'.$hex;

		for($i = 0; $i < strlen($hex); $i += 2)
		$ascii .= chr(base_convert(substr($hex, $i, 2), 16, 10));

		return $ascii;
	}

    /**
     * paramEncrypt function just calles AsciiToHex and base64_encode from inside.
     * 
     * @param string $str_in
     * @return string
     */
	function paramEncrypt($strIn) {
		return $this->AsciiToHex(base64_encode($strIn));
	}

    /**
     * paramDecrypt function just calls AsciiToHex and base64_decode from inside.
     * 
     * @param string $strIn
     * @return string
     */
	function paramDecrypt($strIn) {
		return base64_decode($this->HexToAscii($strIn));
	}
    
    /**
     * Converts array to string
     * 
     * @param array $paramArray
     * @return string
     */
    function arrayToString($paramArray)
    {
         $carryfilters = NULL;
         $first = TRUE;
         foreach($paramArray as $key => $value) {
             if ($first) {$first = FALSE;} else {$carryfilters .= '&';}
             $carryfilters .= $key.'='.$value;
         }
         return $carryfilters;
    }
   
    /**
     * Converts string to array
     * 
     * @param string $strIn
     * @return string $paramArray and false on failure.
     */
    function stringToArray($strIn)
    {
        if (NULL != $strIn) {
            $rawParamArray = explode('&', $strIn);

            foreach ($rawParamArray as $v) {
                if (NULL != $v) {
                    list($key, $value) = preg_split('/=/', $v);
                    $paramArray[$key] = $value;
                }
            }
            return $paramArray;
        }
        return false;
    }
}